/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.fleet.model;

import java.beans.PropertyEditor;
import java.beans.PropertyEditorSupport;
import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import org.openide.nodes.Node;

/**
 * Value holder for machine properties
 *
 * If using possible values in machine passed {@link MachinePropertyDefinition}
 * then implement the toString() on the returned objects. toString is used to
 * supply the possible tags for {@link PropertyEditorSupport#getTags()}
 *
 * @author Merijn Zengers <Merijn.Zengers@gmail.nl>
 */
public class MachineProperty<T> implements Serializable {

    private T value;
    private final MachinePropertyDefinition machinePropertyDefinition;

    public MachineProperty(T value, MachinePropertyDefinition machineProperty) {
        this.value = value;
        this.machinePropertyDefinition = machineProperty;
    }

    public T getValue() {
        return value;
    }

    public final void setValue(T val) throws IllegalAccessException, IllegalArgumentException, InvocationTargetException {
        this.value = val;
    }

    public MachinePropertyDefinition getMachinePropertyDefinition() {
        return machinePropertyDefinition;
    }

    public Node.Property<T> getProperty() {
        return new Node.Property<T>(MachineProperty.this.machinePropertyDefinition.getValueType()) {

            @Override
            public boolean canRead() {
                return true;
            }

            @Override
            public T getValue() throws IllegalAccessException, InvocationTargetException {
                return MachineProperty.this.value;
            }

            @Override
            public boolean canWrite() {
                return true;
            }

            @Override
            public void setValue(T val) throws IllegalAccessException, IllegalArgumentException, InvocationTargetException {
                MachineProperty.this.value = val;
            }

            @Override
            public String getName() {
                return MachineProperty.this.machinePropertyDefinition.getName();
            }

            @Override
            public PropertyEditor getPropertyEditor() {
                final Object[] possibleValues = MachineProperty.this.machinePropertyDefinition.getPossibleValues();
                if (possibleValues == null || possibleValues.length == 0) {
                    return super.getPropertyEditor();
                } else {
                    return new PropertyEditorSupport() {

                        @Override
                        public String[] getTags() {
                            String[] tags = new String[possibleValues.length];
                            for(int i =0; i < possibleValues.length; i++){
                                tags[i] = possibleValues[i].toString();
                            }
                            return tags;
                        }
                    };
                }
            }
        };
    }
}
