/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.fleet.model.gps;

import com.vividsolutions.jts.geom.Point;
import nl.cloudfarming.client.fleet.model.Machine;
import org.openide.util.Exceptions;

/**
 * This class can be use superclass for implementations of GPS trackers. It
 * provides functions for enabling and disabling the tracker. Implementations
 * should override the getPosition() method.
 *
 * @author Wytse Visser
 */
public abstract class GPSTracker implements Runnable {

    private Thread thread = null;
    private Machine machine;
    private boolean enabled = false;
    protected static final int UPDATES_PER_SECOND = 1;
//    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.fleet.model.gps.GPSTracker");

    public void setMachine(Machine machine) {
        this.machine = machine;
    }

    /**
     * Toggles the gps tracker. If it's currently enabled, disable it and the
     * other way around.
     */
    public final void toggle() {
        if (isEnabled()) {
            disable();
        } else {
            enable();
        }
    }

    private void disable() {
        setEnabled(false);
    }

    private void enable() {
        if (thread == null) {
            thread = new Thread(this);
            thread.start();
        }
        setEnabled(true);
    }

    /**
     *
     * @return Whether the gps tracker is enabled.
     */
    protected boolean isEnabled() {
        return enabled;
    }

    private void setEnabled(boolean enabled) {
        this.enabled = enabled;
    }

    /**
     *
     * @return The machine this gps tracker is tracking for.
     */
    protected Machine getMachine() {
        return machine;
    }

    /**
     * This method runs the gps tracker. It starts a loop in which it sleeps for
     * a while (based on UPDATES_PER_SECOND) and then updates the position of
     * the machine.
     */
    @Override
    public void run() {
        while (true) {
            try {
                Thread.sleep(1000 / UPDATES_PER_SECOND);
            } catch (InterruptedException ex) {
                Exceptions.printStackTrace(ex);
            }
            if (isEnabled()) {
                Point position = getPosition();
                getMachine().setLongitude(position.getX());
                getMachine().setLatitude(position.getY());
            }
        }
    }

    /**
     * Get the current position of the machine.
     *
     * @return The current position of the machine.
     */
    protected abstract Point getPosition();
}
